function FPexample()
% Example code for simple FP cavity

%% ===== Define Some Parameters =====
close all
clear all

Pin=1;          % input power
fmod1=9e6;      % modulation frequency
vMod=[-fmod1;0;fmod1];
g1=0.4i;        % modulation depth
opt = Optickle(vMod);
c=opt.c;
h=opt.h;
lambda=opt.lambda(1);
omega=2*pi*c/lambda;
omegamod=2*pi*fmod1;
Nmd=1.45;       % refractive index

% cavity parameters without thermal lensing (for mode-matching)
Larm=5000;      % cavity length
IChr=1/2500;  % ITM curvature
EChr=1/2700;  % ETM curvature
gI=1-Larm*IChr;
gE=1-Larm*EChr;
z=Larm*gE*(1-gI)/(gI+gE-2*gI*gE);
z0=Larm*sqrt(gI*gE*(1-gI*gE))/(gI+gE-2*gI*gE);
q=z+i*z0;
ABCD=inv([1,0;0,1/Nmd])*[1,0;(Nmd-1)/Nmd*IChr,1/Nmd];
q=(ABCD(1,1)*q+ABCD(1,2))/(ABCD(2,1)*q+ABCD(2,2));
lasz=real(q);
lasz0=imag(q);

% cavity parameters with thermal lensing
Larm=5000;      % cavity length
ITMThr=0.02;    % ITM transmission
ETMThr=0.0001;  % ETM transmission
ITMChr=1/2500;  % ITM curvature
ETMChr=1/2700;  % ETM curvature
HALFThr=0.5;    % WFS BS transmission
PICKThr=0.999;   % REFL pitckoff mirror transmission
fTL=Inf;        % focal length of ITM thermal lens
fL1=3;          % focal lengths of Gouy phase telescope
fL2=-1;
LITM_L1=5;      % some lengths
LL1_L2=2.069;
LH_A=14.4908;
LH_B=14.8588;

mTM = 1;
QTM = 10;       % mechanical Q of the TM pendulum
wTM = 2*pi*1;   % pendulum freq.(rad/s)
pendulumTF = zpk([], -wTM*[1/(2*QTM) + sqrt((1/(2*QTM))^2 - 1), 1/(2*QTM) - sqrt((1/(2*QTM))^2 - 1)], 1/mTM);
Ipit = 1;
Qpit = 10;      % mechanical Q of the TM pitch
wpit = 2*pi*3;  % pitch freq.(rad/s)
pitTF = zpk([], -wpit*[1/(2*Qpit) + sqrt((1/(2*Qpit))^2 - 1), 1/(2*Qpit) - sqrt((1/(2*Qpit))^2 - 1)], 1/Ipit);

t1=sqrt(ITMThr);
r1=sqrt(1-ITMThr);
t2=sqrt(ETMThr);
r2=sqrt(1-ETMThr);
gc=t1^2*r2/(1-r1*r2);
Finesse=pi*sqrt(r1*r2)/(1-r1*r2);
tau=2*Larm*Finesse/(pi*c);



%% ===== Construct Optickle Model =====
[opt, n.Laser] = addSource(opt, 'Laser', (vMod==0)*sqrt(Pin),lasz0,-lasz);    % mode-matched to FP without thermal lensing
%[opt, n.Laser] = addSource(opt, 'Laser', (vMod==0)*sqrt(Pin));      % mode-matched to FP automatically
[opt, n.Mod]= addRFmodulator(opt, 'Mod', fmod1, g1);
[opt, n.ThermLens]= addTelescope(opt, 'ThermLens', fTL);
[opt, n.PICK]  = addBeamSplitter(opt, 'PICK', 45, 0, PICKThr, 0, 0, 0, Nmd);

aio=0; Lhr=0; Rar=0; Lmd=0;
[opt, n.ITM]  = addMirror(opt, 'ITM', aio, ITMChr, ITMThr, Lhr , Rar, Lmd, Nmd);
[opt, n.ETM]  = addMirror(opt, 'ETM', aio, ETMChr, ETMThr, Lhr , Rar, Lmd, Nmd);

opt = setMechTF(opt, 'ITM', pendulumTF);
opt = setMechTF(opt, 'ETM', pendulumTF);
opt = setMechTF(opt, 'ITM', pitTF,2);
opt = setMechTF(opt, 'ETM', pitTF,2);

[opt, l.LasertoMod] = addLink(opt, n.Laser, 'out', n.Mod, 'in',0);
[opt, l.ModtoPICK] = addLink(opt, n.Mod, 'out', n.PICK, 'frA',0);
[opt, l.PICKtoThermLens] = addLink(opt, n.PICK, 'bkA', n.ThermLens, 'in',0);
[opt, l.ThermLenstoITM] = addLink(opt, n.ThermLens, 'out', n.ITM, 'bk',0);
[opt, l.ITMtoETM] = addLink(opt, n.ITM, 'fr', n.ETM, 'fr', Larm);
[opt, l.ETMtoITM] = addLink(opt, n.ETM, 'fr', n.ITM, 'fr', Larm);

% TRANS port
[opt, n.TRANS] = addSink(opt, 'TRANS');
[opt, l.ETMtoTRANS] = addLink(opt, n.ETM, 'bk', n.TRANS, 'in', 0);
[opt, pr.TRANS] = addProbeIn(opt, 'TRANS_DC', n.TRANS, 'in', 0, 0);

% REFL port
[opt, n.L1]= addTelescope(opt, 'L1', fL1);
[opt, n.L2]= addTelescope(opt, 'L2', fL2);
[opt, n.ThermLens2]= addTelescope(opt, 'ThermLens2', fTL);
[opt, n.AttREFL] = addSink(opt, 'AttREFL',0);
[opt, n.REFLHALF]  = addMirror(opt, 'REFLHALF', 45, 0, HALFThr, 0, 0, 0, Nmd);
[opt, n.REFLA] = addSink(opt, 'REFLA');
[opt, n.REFLB] = addSink(opt, 'REFLB');
[opt, n.GouyREFLA] = addGouyPhase(opt, 'GouyREFLA', 0);
[opt, n.GouyREFLB] = addGouyPhase(opt, 'GouyREFLB', 0);
[opt, l.ITMtoThermLens] = addLink(opt, n.ITM, 'bk', n.ThermLens2, 'in', 0);
[opt, l.ThermLenstoPICK] = addLink(opt, n.ThermLens2, 'out', n.PICK, 'bkB', 0);
[opt, l.PICKtoL1] = addLink(opt, n.PICK, 'bkB', n.L1, 'in', LITM_L1);
[opt, l.L1toL2] = addLink(opt, n.L1, 'out', n.L2, 'in', LL1_L2);
[opt, l.L2toAttREFL] = addLink(opt, n.L2, 'out', n.AttREFL, 'in', 0);
[opt, l.AttREFLtoREFLHALF] = addLink(opt, n.AttREFL, 'out', n.REFLHALF,'fr',0);
[opt, l.REFLHALFtoGouyREFLA] = addLink(opt, n.REFLHALF, 'fr', n.GouyREFLA, 'in',LH_A);
[opt, l.GouyREFLAtoREFLA] = addLink(opt, n.GouyREFLA, 'out', n.REFLA, 'in',0);
[opt, l.REFLHALFtoGouyREFLB] = addLink(opt, n.REFLHALF, 'bk', n.GouyREFLB, 'in',LH_B);
[opt, l.GouyREFLBtoREFLB] = addLink(opt, n.GouyREFLB, 'out', n.REFLB, 'in',0);

demodphaseA=-mod((2*Larm+LITM_L1+LL1_L2+LH_A)*omegamod/c/pi*180,360);       % calculated
demodphaseB=-mod((2*Larm+LITM_L1+LL1_L2+LH_B)*omegamod/c/pi*180,360);       % calculated
[opt, pr.REFL_A1I] = addProbeIn(opt, 'REFL_A1I', n.REFLA, 'in', fmod1, demodphaseA);
[opt, pr.REFL_A1Q] = addProbeIn(opt, 'REFL_A1Q', n.REFLA, 'in', fmod1, demodphaseA+90);
[opt, pr.REFL_B1I] = addProbeIn(opt, 'REFL_B1I', n.REFLB, 'in', fmod1, demodphaseB);
[opt, pr.REFL_B1Q] = addProbeIn(opt, 'REFL_B1Q', n.REFLB, 'in', fmod1, demodphaseB+90);

% tell Optickle to use this cavity basis
opt = setCavityBasis(opt, 'ITM', 'ETM');

vDist = getLinkLengths(opt);
vBasis = getAllFieldBases(opt);
vPhiGouy = getGouyPhase(vDist, vBasis(:, 2)) * 180 / pi;

%opt2dot(opt, 'FPcomm.dot');

%% Check power at each DC probe, field evaluation point
[fDC, sigDC] = tickle(opt, [], []);
power=@(sn) abs(fDC(sn,round(length(vMod)/2),1)).^2;
spower=@(sn) abs(fDC(sn,round(length(vMod)/2)-1,1)).^2;
fprintf('Laser output: \t %g W \n', power(l.LasertoMod));
fprintf('Cavity input power: \t %g W \n', power(l.ThermLenstoITM));
fprintf('Cavity input power (sideband): \t %g W \n', spower(l.ThermLenstoITM));
fprintf('Intra-cavity power: \t %g W \n', power(l.ITMtoETM));
fprintf('Intra-cavity power: \t %g W \n', power(l.ETMtoITM));
fprintf('Power at TRANS: \t %g mW \n', power(l.ETMtoTRANS)*1e3);
fprintf('Power at REFL_A: \t %g mW \n', power(l.GouyREFLAtoREFLA)*1e3);
fprintf('Power at REFL_A(sideband): \t %g mW \n', spower(l.GouyREFLAtoREFLA)*1e3);
fprintf('Power at REFL_B: \t %g mW \n', power(l.GouyREFLBtoREFLB)*1e3);
fprintf('Power at REFL_B(sideband): \t %g mW \n', spower(l.GouyREFLBtoREFLB)*1e3);


%% ===== Sweep =====
drvNames=getDriveNames(opt);
Ndrive=opt.Ndrive;

Nstep = 1001;
range=lambda/20;
dl=linspace(-range,range,Nstep);
pos=zeros(Ndrive, Nstep);
pos(getDriveIndex(opt,'ETM'),:) = dl;
[fDC, sigDC]=sweep(opt,pos);

% analytical calculation
phi = @(omega,dl) omega*dl./c;
refl =@(phi) -r1+(t1^2*r2*exp(2i*phi))./(1-r1*r2*exp(2i*phi));
trans=@(phi) (t1*t2*exp(i*phi))./(1-r1*r2*exp(2i*phi));
phiomega=phi(omega,dl);
phiomegaplus=phi(omega,dl)+phi(omegamod,Larm+dl);
phiomegaminus=phi(omega,dl)-phi(omegamod,Larm+dl);
Pttheory=Pin*(abs(besselj(0,g1/1i)*trans(phiomega)).^2+abs(besselj(1,g1/1i)*trans(phiomegaplus)).^2+abs(besselj(1,g1/1i)*trans(phiomegaminus)).^2);
Pmodtheory=PICKThr*(1-PICKThr)*HALFThr*Pin*besselj(0,g1/1i)*besselj(1,g1/1i)*imag(refl(phiomega).*conj(refl(phiomegaplus))-conj(refl(phiomega)).*refl(phiomegaminus));

% plot PDH signal
figure();
plot(dl/lambda, squeeze(sigDC(pr.TRANS,:)), '.', dl/lambda, squeeze(sigDC(pr.REFL_A1I,:)), '.', dl/lambda, Pttheory, dl/lambda, Pmodtheory);
xlabel('Displacement [\lambda]');
ylabel('Signal [W]');
title('ETM Sweep');
grid on
legend({'TRANS_DC','REFL_A1I','TRANS calc','REFL calc'},'Interpreter','none');
saveas(gcf,'PDH.png')



%% ===== Tickle =====
freq = logspace(-2,8,100);
[fDC, sigDC, sigAC, mMech, noiseAC] = tickle(opt, [], freq);

% analytical calculation
rs=-real(refl(phi(omegamod,Larm)));
fcp=1/(2*pi*tau);
theory=PICKThr*(1-PICKThr)*HALFThr*8*pi*Pin*besselj(0,g1/1i)*besselj(1,g1/1i)*(t1^2*r2*rs)/(1-r1*r2)^2/lambda*fcp./(fcp+i*freq);

% plot frequency response of FP cavity
figure();
resp=sigAC(pr.REFL_A1I, getDriveIndex(opt,'ETM'),:);
resp = reshape(resp,1,[]);
subplot(2,1,1);
loglog(freq,abs(resp),'.',freq,abs(theory));
ylabel('Abs [W/m]');
title('ETM tickle, REFL response');
legend('tickle','calc',3);
grid on;
subplot(2,1,2);
semilogx(freq,angle(-1*resp)*180/pi,'.',freq,angle(theory)*180/pi,freq,angle(-theory)*180/pi);
legend('tickle','calc Optickle 1','calc Optickle 2',3);
xlabel('Frequency [Hz]');
ylabel('Phase [deg]');
grid on;
ylim([-180,180]);
set(gca,'YTick',linspace(-180,180,7));
saveas(gcf,'FPresponse.png')



%% ===== Tickle01 =====
gouystep=30;
wfs=zeros(2, gouystep);
phi=linspace(0,3*pi,gouystep);
for k=1:gouystep
    opt = setGouyPhase(opt, 'GouyREFLA', phi(k));
    [sigAC, mMech] = tickle01(opt, [], 0.1);
    wfs(:,k)=sigAC(pr.REFL_A1I, [getDriveIndex(opt,'ITM');getDriveIndex(opt,'ETM')],:);
end

%% analytical calculation
k=2*pi/lambda;
w0=sqrt(abs(2*imag(vBasis(l.ITMtoETM))/k));
alpha0=2/(k*w0);
rc = @(Phi) -r1+(t1^2*r2*exp(i*Phi))./(1-r1*r2*exp(i*Phi));
rc0 = rc(0);
rc1 = real(rc(-vPhiGouy(l.ITMtoETM)*2));    % approximately real
rs0= -1;
rs1= -1;
A=PICKThr*(1-PICKThr)*HALFThr*sqrt(8/pi)*Pin*besselj(0,g1/1i)*besselj(1,g1/1i)*(rc0*rs1-rc1*rs0);
gF=1-Larm*ITMChr;
gE=1-Larm*ETMChr;
dxF=-Larm*gE/(2*gF*gE-gF-gE);
dpF=(1-gE)/(1-gF*gE);
dxE=-Larm*gF/(2*gF*gE-gF-gE);
dpE=-(1-gF)/(1-gF*gE);

phit=linspace(0,3*pi,gouystep*10);
ofs=-angle(vBasis(l.ETMtoITM))+(vPhiGouy(l.PICKtoL1)+vPhiGouy(l.L1toL2)+vPhiGouy(l.REFLHALFtoGouyREFLA))/180*pi;  % Gouy phase from the cavity waist to REFLA
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% TREATMENT OF GOUY PHASE DEPENDS ON OPTICKLE VERSION ?
ft =@(phit) A*(dxF/w0.*sin(phit-ofs)-dpF/alpha0.*cos(phit-ofs));
et =@(phit) A*(dxE/w0.*sin(phit-ofs)-dpE/alpha0.*cos(phit-ofs));
fronttheory=ft(phit);
endtheory=et(phit);
mSenstheory=[ft(0),et(0);0,0;ft(pi/2),et(pi/2);0,0;];

% plot WFS signal vs Gouy phase
figure();
plot(phi/pi, real(wfs(1,:)),'.', phi/pi, real(wfs(2,:)),'.',phit/pi,fronttheory/sqrt(2/pi),phit/pi,endtheory/sqrt(2/pi),phit/pi,fronttheory,phit/pi,endtheory);
legend('ITM Optickle','ETM Optickle','ITM analytical/sqrt(2/pi)', 'ETM analytical/sqrt(2/pi)','ITM analytical', 'ETM analytical',4);

xlabel('Gouy phase [rad/\pi]');
ylabel('WFS signal [W/rad]');
grid on;
saveas(gcf,'WFSgouy.png')



%% ===== WFS Sensing Matrix =====
opt = setGouyPhase(opt, 'GouyREFLA', 0);
opt = setGouyPhase(opt, 'GouyREFLB', 0);
[sigAC, mMech]=tickle01(opt,[],1);

% Sensing matrix
figure();
ax=axes();
driveNames={'ITM','ETM'};
probeNames = {'REFL_A1I','REFL_A1Q','REFL_B1I','REFL_B1Q'};
iPrb=cellfun(@(x)getProbeNum(opt,x),probeNames);
iDrv=cellfun(@(x)getDriveIndex(opt,x),driveNames);
mSens=sigAC(iPrb,iDrv);

Nprb=length(probeNames);
Ndrv=length(driveNames);
% Make grid
for ii=0:Nprb
    line([0,Ndrv],[ii,ii],'Color','k');
end
for ii=0:Ndrv
    line([ii,ii],[0,Nprb],'Color','k');
end
set(ax, 'XLim',[0,Ndrv],'YLim',[0,Nprb]);
set(ax,'XTick',[0.5:Ndrv-0.5], 'YTick', [0.5:Nprb-0.5]);
set(ax,'XTickLabel',driveNames,'YTickLabel',probeNames);

for ii=1:Nprb
    [m,maxjj]=max(abs(mSens(ii,:)));
    if m~= 0
        for jj=1:Ndrv
            ss=abs(mSens(ii,jj))*sign(real(mSens(ii,jj)));
            st=abs(mSenstheory(ii,jj))*sign(real(mSenstheory(ii,jj)));
            text(jj-0.5,ii-0.7,['opt  : ', num2str(ss, '%.2f')], 'HorizontalAlignment','center','color',[abs(mSens(ii,jj))/m,0,0]);
            text(jj-0.5,ii-0.5,['calc/sqrt(2/pi) : ', num2str(st/sqrt(2/pi), '%.2f')], 'HorizontalAlignment','center');
            text(jj-0.5,ii-0.3,['calc : ',num2str(st, '%.2f')], 'HorizontalAlignment','center');
        end
    end
end

title('WFS Sensing Matrix [W/rad]','FontSize',16);
set(gca,'XAxisLocation','top'); 
set(gca,'YDir','reverse');
fnote=strcat('(Gouy phases at REFL A: ',num2str(vPhiGouy(l.REFLHALFtoGouyREFLA),'%.1f'),', REFL B: ',num2str(vPhiGouy(l.REFLHALFtoGouyREFLB),'%.1f'),' deg)');
xlabel(fnote,'FontSize',10)

saveas(gcf,'WFSmatrix.png')



%% ===== Diagonalized WFS Matrix =====
opt = setGouyPhase(opt, 'GouyREFLA', 0);
opt = setGouyPhase(opt, 'GouyREFLB', 0);
[sigAC, mMech]=tickle01(opt,[],1);

% Sensing matrix
figure();
ax=axes();
driveNames={'ITM','ETM'};
probeNames = {'REFL_A1I','REFL_B1I'};
iPrb=cellfun(@(x)getProbeNum(opt,x),probeNames);
iDrv=cellfun(@(x)getDriveIndex(opt,x),driveNames);
mInput=[0.1013e-1,-0.0995e-1;0.0509e-1,-0.1206e-1]; % Original input matrix
mSens=mInput*sigAC(iPrb,iDrv);

Nprb=length(probeNames);
Ndrv=length(driveNames);
% Make grid
for ii=0:Nprb
    line([0,Ndrv],[ii,ii],'Color','k');
end
for ii=0:Ndrv
    line([ii,ii],[0,Nprb],'Color','k');
end
set(ax, 'XLim',[0,Ndrv],'YLim',[0,Nprb]);
set(ax,'XTick',[0.5:Ndrv-0.5], 'YTick', [0.5:Nprb-0.5]);
set(ax,'XTickLabel',driveNames,'YTickLabel',probeNames);

for ii=1:Nprb
    [m,maxjj]=max(abs(mSens(ii,:)));
    if m~= 0
        for jj=1:Ndrv
            ss=abs(mSens(ii,jj))*sign(real(mSens(ii,jj)));
            text(jj-0.5,ii-0.5,num2str(ss, '%.2f'), 'HorizontalAlignment','center','color',[abs(mSens(ii,jj))/m,0,0]);
        end
    end
end

title('Diagonalized WFS Matrix [rad/rad]','FontSize',16);
set(gca,'XAxisLocation','top'); 
set(gca,'YDir','reverse');

saveas(gcf,'WFSmatrix_diag.png')

